@@ -27,7 +27,6 @@ Notes:
 #include "ast/ast_smt2_pp.h"
 #include "tactic/tactic.h"
 #include "tactic/tactical.h"
-#include "tactic/probe.h"
 #include "solver/check_sat_result.h"
 #include "cmd_context/cmd_context_to_goal.h"
 #include "cmd_context/echo_tactic.h"
@@ -38,9 +37,6 @@ probe_info::probe_info(symbol const & n, char const * d, probe * p):
     m_probe(p) {
 }
 
-probe_info::~probe_info() {
-}
-
 class declare_tactic_cmd : public cmd {
     symbol           m_name;
     sexpr *          m_decl;
@@ -231,8 +227,8 @@ public:
                 }
                 catch (z3_exception & ex) {
                     result->set_status(l_undef);
-                    result->m_unknown = ex.msg();
-                    ctx.regular_stream() << "(error \"tactic failed: " << ex.msg() << "\")" << std::endl;
+                    result->m_unknown = ex.what();
+                    ctx.regular_stream() << "(error \"tactic failed: " << ex.what() << "\")" << std::endl;
                 }
                 ctx.validate_check_sat_result(r);
             }
@@ -325,7 +321,7 @@ public:
                     exec(t, g, result_goals);
                 }
                 catch (tactic_exception & ex) {
-                    ctx.regular_stream() << "(error \"tactic failed: " << ex.msg() << "\")" << std::endl;
+                    ctx.regular_stream() << "(error \"tactic failed: " << ex.what() << "\")" << std::endl;
                     failed = true;
                 }
             }
