@@ -19,7 +19,7 @@ Notes:
 
 #include "sat/sat_local_search.h"
 #include "sat/sat_solver.h"
-#include "sat/sat_params.hpp"
+#include "params/sat_params.hpp"
 #include "util/timer.h"
 
 namespace sat {
@@ -353,16 +353,13 @@ namespace sat {
         DEBUG_CODE(verify_unsat_stack(););
     }
 
-    local_search::local_search() {
-    }
-
     void local_search::reinit(solver& s, bool_vector const& phase) {
         import(s, true);
         for (unsigned i = phase.size(); i-- > 0; )
             set_phase(i, phase[i]);
     }
 
-    void local_search::import(solver const& s, bool _init) {        
+    void local_search::import(solver const& s, bool _init) {
         flet<bool> linit(m_initializing, true);
         m_is_pb = false;
         m_vars.reset();
@@ -412,16 +409,13 @@ namespace sat {
             [&](unsigned sz, literal const* c, unsigned k) { add_cardinality(sz, c, k); };
         std::function<void(unsigned sz, literal const* c, unsigned const* coeffs, unsigned k)> pb = 
             [&](unsigned sz, literal const* c, unsigned const* coeffs, unsigned k) { add_pb(sz, c, coeffs, k); };
-        if (ext && (!ext->is_pb() || !ext->extract_pb(card, pb)))
+        if (ext && (!ext->is_pb() || !ext->extract_pb(card, pb))) {
+            IF_VERBOSE(0, verbose_stream() << (ext) << " is-pb " << (!ext && ext->is_pb()) << "\n");
             throw default_exception("local search is incomplete with extensions beyond PB");
-        
+        }
         if (_init) 
             init();        
     }
-    
-    local_search::~local_search() {
-    }
-    
 
     lbool local_search::check() {
         return check(0, nullptr, nullptr);
