@@ -1396,8 +1396,7 @@ namespace smt {
             DEBUG_CODE(for (literal lit : simp_lits) SASSERT(get_assignment(lit) == l_true););
             if (!simp_lits.empty()) {
                 j = mk_justification(unit_resolution_justification(*this, j, simp_lits.size(), simp_lits.data()));
-            }
-              
+            }            
             break;
         }
         case CLS_TH_LEMMA:
@@ -1423,6 +1422,7 @@ namespace smt {
         unsigned activity = 1;
         bool  lemma = is_lemma(k);
         m_stats.m_num_mk_lits += num_lits;
+
         switch (num_lits) {
         case 0:
             if (j && !j->in_region())
@@ -1431,12 +1431,16 @@ namespace smt {
             set_conflict(j == nullptr ? b_justification::mk_axiom() : b_justification(j));
             SASSERT(inconsistent());
             return nullptr;
-        case 1:
+        case 1: {
+            literal unit = lits[0];
+            expr* atom = m_bool_var2expr[unit.var()];
             if (j && !j->in_region())
                 m_justifications.push_back(j);
-            assign(lits[0], j);
-            inc_ref(lits[0]);
+            assign(unit, j);
+            inc_ref(unit);
+            // m_units_to_reassert.push_back({ expr_ref(atom, m), unit.sign(), is_relevant(unit) });
             return nullptr;
+        }
         case 2:
             if (use_binary_clause_opt(lits[0], lits[1], lemma)) {
                 literal l1 = lits[0];
@@ -1562,6 +1566,10 @@ namespace smt {
         mk_clause(num_lits, lits, js, k);
     }
     
+    void context::mk_th_axiom(theory_id tid, literal l1, unsigned num_params, parameter * params) {
+        mk_th_axiom(tid, 1, &l1, num_params, params);
+    }
+    
     void context::mk_th_axiom(theory_id tid, literal l1, literal l2, unsigned num_params, parameter * params) {
         literal ls[2] = { l1, l2 };
         mk_th_axiom(tid, 2, ls, num_params, params);
@@ -1830,7 +1838,7 @@ namespace smt {
             // Case) there is a variable old_v in the var-list of n.
             //
             // Remark: This variable was moved to the var-list of n due to a add_eq.
-            SASSERT(th->get_enode(old_v) != n); // this varialbe is not owned by n
+            SASSERT(th->get_enode(old_v) != n); // this variable is not owned by n
             SASSERT(n->get_root()->get_th_var(th_id) != null_theory_var); // the root has also a variable in its var-list.
             n->replace_th_var(v, th_id);
             push_trail(replace_th_var_trail( n, th_id, old_v));
