@@ -959,7 +959,7 @@ namespace smt {
             return internalize_term_core(term);
         }
         catch (z3_exception& ex) {
-            IF_VERBOSE(1, verbose_stream() << "internalize_term: " << ex.msg() << "\n";);
+            IF_VERBOSE(1, verbose_stream() << "internalize_term: " << ex.what() << "\n";);
             throw;
         }
     }
@@ -1493,9 +1493,6 @@ namespace smt {
         m_bb.set_flat_and_or(false);
     }
 
-    theory_bv::~theory_bv() {
-    }
-
     theory* theory_bv::mk_fresh(context* new_ctx) {
         return alloc(theory_bv, *new_ctx);
     }
@@ -1789,6 +1786,27 @@ namespace smt {
         return false;
     }
 
+    void theory_bv::initialize_value(expr* var, expr* value) {
+        rational val;
+        unsigned sz;
+        TRACE("bv", tout << "initializing " << mk_pp(var, m) << " := " << mk_pp(value, m) << "\n");
+        if (!m_util.is_numeral(value, val, sz)) {
+            IF_VERBOSE(5, verbose_stream() << "value should be a bit-vector " << mk_pp(value, m) << "\n");
+            return;
+        }
+        if (!is_app(var))
+            return;
+        enode* n = mk_enode(to_app(var));
+        auto v = get_var(n);
+        unsigned idx = 0;
+        for (auto lit : m_bits[v]) {
+            auto & b = ctx.get_bdata(lit.var());
+            b.m_phase_available = true;
+            b.m_phase = val.get_bit(idx);
+            ++idx;
+        }
+    }
+
     void theory_bv::init_model(model_generator & mg) {
         m_factory = alloc(bv_factory, m);
         mg.register_factory(m_factory);
@@ -1967,6 +1985,7 @@ namespace smt {
             while (curr != v);
 
             zero_one_bits const & _bits = m_zero_one_bits[v];
+            (void)num_bits;
             SASSERT(_bits.size() == num_bits);
             bool_vector already_found;
             already_found.resize(bv_sz, false);
